import uiza
from uiza import Connection
from uiza.api_resources.base.base import UizaBase
from uiza.settings.config import settings
from uiza.utility.utility import set_url


class User(UizaBase):

    def __init__(self):
        self.connection = Connection(workspace_api_domain=uiza.workspace_api_domain, api_key=uiza.authorization)
        self.connection.url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.user.type,
            api_version=settings.uiza_api.user.version,
            api_sub_url=settings.uiza_api.user.sub_url
        )

    def change_password(self, id, old_password, new_password):
        """
        Change password allows Admin or User update their current password.
        :param id: identifier of user need reset password
        :param old_password: current password
        :param new_password: new password (from a to Z, 6 to 25 characters)
        :return: tuple of id of user and status code
        """
        self.connection.url = '{}/changepassword'.format(self.connection.url)
        data_body = dict(
            id=id,
            oldPassword=old_password,
            newPassword=new_password
        )
        data = self.connection.put(data=data_body)

        return data

    def logout(self):
        """
        Log out an user
        :return: message
        """
        self.connection.url = '{}/logout'.format(self.connection.url)
        data = self.connection.post()

        return data



import unittest
import mock
import uiza

from uiza.api_resources.storage import Storage
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError,
    ClientException
)


class TestStorageBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestStorageBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.authorization = 'test api key'
        self.storage_id = '37d6706e-be91-463e-b3b3-b69451dd4752'
        self.storage_data_create = {
            'name': 'FTP Uiza test 4',
            'description': 'FTP of Uiza, use for transcode',
            'storageType': 'ftp',
            'host': 'ftp-example.uiza.io'
        }

        self.storage_data_update = {
            'id': '37d6706e-be91-463e-b3b3-b69451dd4752',
            'name': 'FTP Uiza test 5',
            'description': 'FTP of Uiza, use for transcode',
            'storageType': 'ftp',
            'host': 'ftp-example.uiza.io'
        }


class TestAddStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Storage().add(**self.storage_data_create)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_add_storage_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Storage().add(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRetrieveStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Storage().retrieve(id=self.storage_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Storage().retrieve(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_storage_invalid_with_not_storage_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            Storage().retrieve()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdateStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Storage().update(**self.storage_data_update)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Storage().update(**self.storage_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRemoveStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Storage().remove(id=self.storage_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Storage().remove(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_remove_storage_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Storage().remove()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestListStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_storage_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Storage().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestCreateStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_storage_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Storage().create(**self.storage_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')

class TestDeleteStorage(TestStorageBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_storage_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Storage().delete(id=self.storage_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')



import unittest
import mock
import uiza

from uiza.api_resources.user import User
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError
)


class TestUserBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestUserBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.authorization = 'test api key'
        self.user_id = '37d6706e-be91-463e-b3b3-b69451dd4752'
        self.user_data_create = {
            'status': 1,
            'username': 'user_test',
            'email': 'user_test@uiza.io',
            'fullname': 'User Test',
            'avatar': 'https://exemple.com/avatar.jpeg',
            'dob': '05/15/2018',
            'gender': 0,
            'password': 'FMpsr<4[dGPu?B#u',
            'isAdmin': 1
        }
        self.user_data_update = dict(id=self.user_id, name='Test update')


class TestCreateUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().create(**self.user_data_create)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().create(**self.user_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestListUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().list()
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRetrieveUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().retrieve(id=self.user_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().retrieve(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_user_invalid_with_not_user_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            User().retrieve()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdateUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().update(**self.user_data_update)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().update(**self.user_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestDeleteUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().delete(id=self.user_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().delete(id=self.user_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_user_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            User().delete()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdatePasswordUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().change_password(
            id = self.user_id,
            new_password='New password',
            old_password='Old password'
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_password_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().change_password(
                id=self.user_id,
                new_password='New password',
                old_password='Old password'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestLogoutUser(TestUserBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = User().logout()
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_logout_user_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            User().logout()
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')



import uiza
from uiza import Connection
from uiza.api_resources.base.base import UizaBase
from uiza.settings.config import settings
from uiza.utility.utility import set_url
from uiza.exceptions import ClientException


class Live(UizaBase):

    def __init__(self):
        self.connection = Connection(workspace_api_domain=uiza.workspace_api_domain, api_key=uiza.authorization)
        self.connection.url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.livestreaming.type,
            api_version=settings.uiza_api.livestreaming.version,
            api_sub_url=settings.uiza_api.livestreaming.sub_url
        )

    def list(self, **params):
        """
        Override method list of Uizabase
        :param params:
        :return: Raise error when get method list
        """
        raise ClientException('Livestreaming list method not found')

    def start_feed(self, id):
        """
        Start a live event
        :param id: identifier of event.
        :return: tuple of id event and status code
        """
        self.connection.url = '{}/feed'.format(self.connection.url)
        result = self.connection.post(dict(id=id))

        return result

    def stop_feed(self, id):
        """
        Stop a live event
        :param id: identifier of event.
        :return: tuple of id event and status code
        """
        self.connection.url = '{}/feed'.format(self.connection.url)
        result = self.connection.put(dict(id=id))

        return result

    def get_view(self,  id):
        """
        Get a live view status
        :param id: event id has been created
        :return: tuple of response and status code
        """
        self.connection.url = '{}/tracking/current-view'.format(self.connection.url)
        query = self.url_encode(params={'id': id})
        result = self.connection.get(query=query)

        return result

    def list_recorded(self):
        """
        List of recorded file after streamed
        :return: tuple of response and status code
        """
        self.connection.url = '{}/dvr'.format(self.connection.url)
        result = self.connection.get()

        return result

    def convert_into_vod(self, id):
        """
        Convert recorded file into VOD entity
        :param id: identifier of record (get from list record)
        :return: tuple of response and status code
        """
        self.connection.url = '{}/dvr/convert-to-vod'.format(self.connection.url)
        result = self.connection.post(dict(id=id))

        return result

    def delete(self, id):
        """
        Delete a recorded file
        :param id: identifier of record (get from list record)
        :return: tuple of id record has been deleted and status code
        """
        self.connection.url = '{}/dvr'.format(self.connection.url)
        result = self.connection.delete(dict(id=id))

        return result



import unittest
import mock
import uiza

from uiza.api_resources.category import Category
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError,
    ClientException
)


class TestCategoryBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestCategoryBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.authorization = 'test api key'
        self.category_id = '37d6706e-be91-463e-b3b3-b69451dd4752'
        self.category_data_create = {
            'name':'Folder sample python 3',
            'type':'tag',
            'description':'Folder description',
            'orderNumber':1,
            'icon':'https://exemple.com/icon.png'
        }
        self.category_data_update = dict(id=self.category_id, name='Test update')


class TestCreateCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().create(**self.category_data_create)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().create(**self.category_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRetrieveCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().retrieve(id=self.category_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().retrieve(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_category_invalid_with_not_category_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            Category().retrieve()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdateCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().update(**self.category_data_update)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().update(**self.category_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestDeleteCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().delete(id=self.category_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().delete(id=self.category_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_category_invalid_not_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Category().delete()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestListCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().list()
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestCreateRelationCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().create_relation(
            entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
            metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_not_entity_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Category().create_relation(
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_relation_category_invalid_not_metadata_ids(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Category().create_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestDeleteRelationCategory(TestCategoryBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Category().delete_relation(
            entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
            metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_not_entity_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Category().delete_relation(
                metadata_ids=['29f7b6ba-e2a7-4d4b-8026-30828d0a1bb0']
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_relation_category_invalid_not_metadata_ids(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Category().delete_relation(
                entity_id='eb578480-6311-4534-b00e-7c7ffbce8283',
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')



import re
import json

try:
    from urllib.request import Request, urlopen
    from urllib.error import HTTPError
except ImportError:
    from urllib2 import Request, urlopen
    from urllib2 import HTTPError

from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError,
    ServerBaseErrors
)


class ResponseObject(object):
    dict = None  # type: dict

    def __init__(self, d):
        self.dict = d
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [ResponseObject(x) if isinstance(x, dict) else x for x in b])
            else:
                setattr(self, a, ResponseObject(b) if isinstance(b, dict) else b)


class Connection(object):
    url = None

    def __init__(self, workspace_api_domain, api_key, **kwargs):
        """
        Handle the Uiza connection
        :param workspace_api_domain: workspace api domain of uiza account
        :param api_key: private key of uiza account
        :param kwargs:
        """
        self.auth_code = api_key
        self.headers = self._set_headers()
        self.workspace_api_domain = workspace_api_domain

    def get(self, query=''):
        """
        Handle request GET
        :param query: params of request GET
        :return: obj and status code after request
        """
        url = self._make_url(query)
        response_data, status_code = self._request_http(url)
        status_code = self._filter_error_status_code(status_code=status_code, response_data=response_data)
        data = {}
        if response_data:
            data = self._get_response_obj(response_data=response_data)
        return data, status_code

    def post(self, data=None):
        """
        Handle request POST
        :param data: data body of request POST
        :return:
        """
        url = self._make_url_with_data(data=data, method='POST')
        response_data, status_code = self._request_http(url)
        status_code = self._filter_error_status_code(status_code=status_code, response_data=response_data)
        data = {}
        if response_data:
            data = self._get_response_obj(response_data=response_data)
        return data, status_code

    def put(self, data):
        """
        Handle request PUT
        :param data: data body of request PUT
        :return: obj and status code after request
        """
        url = self._make_url_with_data(data=data, method='PUT')
        response_data, status_code = self._request_http(url)
        status_code = self._filter_error_status_code(status_code=status_code, response_data=response_data)
        if response_data:
            data = self._get_response_obj(response_data=response_data)
        return data, status_code

    def delete(self, data):
        """
        Handle request DELETE
        :param data: data body of request DELETE
        :return: obj and status code after request
        """
        url = self._make_url_with_data(data=data, method='DELETE')
        response_data, status_code = self._request_http(url)
        status_code = self._filter_error_status_code(status_code=status_code, response_data=response_data)
        data = {}
        if response_data:
            data = self._get_response_obj(response_data=response_data)
        return data, status_code

    def _make_url(self, query):
        """
        Make Request urllib/urllib2 with query
        :param query: query of method GET
        :return: Request object
        """
        return Request('{}{}'.format(self.url, query), headers=self.headers)

    def _make_url_with_data(self, method, data=None):
        """
        Make Request urllib/urllib2 with body data
        :param method: method of request. It include POST, PUT, DELETE
        :param data: boday data of request
        :return: Request object
        """
        req_data = None
        if data:
            req_data = json.dumps(data).encode('utf8')
        try:
            request = Request(self.url, data=req_data, headers=self.headers, method=method)
        except TypeError:
            request = Request(self.url, data=req_data, headers=self.headers)
            request.get_method = lambda: method

        return request

    def _set_headers(self):
        """
        Set header of request
        :return:
        """
        headers = {'Content-Type': 'application/json'}
        if self.auth_code:
            auth = {'Authorization': self.auth_code}
            headers.update(auth)

        return headers

    def _request_http(self, url):
        """
        Start request to Uiza server
        :param url: Request obj
        :return: string response, status code
        """
        message_error = None
        try:
            response = urlopen(url)
            try:
                status_code = response.status
            except AttributeError:
                status_code = response.code

            response_data = response.read().decode('utf-8')
        except HTTPError as e:
            try:
                status_code = e.status
            except AttributeError:
                status_code = e.code

            response_data = e.read().decode('utf-8')

        return response_data, status_code

    def _filter_error_status_code(self, status_code, response_data):
        """
        Filter error code if status code in [400, 401, 404, 422, 500, 503, 4xx, 5xx]
        :param status_code: status code
        :param response_data: response
        :return:
        """
        if status_code == 200:
            return status_code

        message_error = None
        response_obj = self._get_message_error(response_data)
        if response_obj:
            message_error = response_obj.get('message')

        if status_code == 400:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_BAD_REQUEST
            raise BadRequestError(message_error)
        elif status_code == 401:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_UNAUTHORIZED
            raise UnauthorizedError(message_error)
        elif status_code == 404:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_NOT_FOUND
            raise NotFoundError(message_error)
        elif status_code == 422:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_UNPROCESSABLE
            raise UnprocessableError(message_error)
        elif status_code == 500:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_INTERNAL_SERVER_ERROR
            raise InternalServerError(message_error)
        elif status_code == 503:
            if not message_error:
                message_error = ServerBaseErrors.ERR_UIZA_SERVICE_UNAVAILABLE
            raise ServiceUnavailableError(message_error)
        elif re.match(r'^4[0-9]{2}$', str(status_code)):
            raise ClientError(ServerBaseErrors.ERR_UIZA_CLIENT_ERROR)
        elif re.match(r'^5[0-9]{2}$', str(status_code)):
            raise ServerError(ServerBaseErrors.ERR_UIZA_SERVER_ERROR)

        return status_code

    def _get_response_obj(self, response_data):
        """
        Convert from string to object of response
        :param response_data: string response
        :return: dict null, object or list object
        """
        try:
            response_dict = json.loads(response_data)
            data = response_dict.get('data')
            if not data:
                return {}

            if isinstance(data, dict):
                return ResponseObject(response_dict.get('data'))

            if isinstance(data, list):
                return [ResponseObject(item) for item in data]
        except Exception:
            return {}

    def _get_message_error(self, response_data):
        """
        Get message error from response data
        :param response_data: string response
        :return: dict
        """
        try:
            response_dict = json.loads(response_data)
            return response_dict
        except Exception:
            return {}



import unittest
import mock
import uiza

from uiza.api_resources.callback import Callback
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError,
    ClientException
)


class TestCallbackBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestCallbackBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.authorization = 'test api key'
        self.callback_id = '37d6706e-be91-463e-b3b3-b69451dd4752'
        self.callback_data_create = {
            "url": "https://callback-url-python.uiza.co",
            "method": "GET"
        }
        self.callback_data_update = dict(id=self.callback_id, name='Test update')


class TestCreateCallback(TestCallbackBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Callback().create(**self.callback_data_create)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_callback_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Callback().create(**self.callback_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRetrieveCallback(TestCallbackBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Callback().retrieve(id=self.callback_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Callback().retrieve(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_callback_invalid_with_not_callback_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            Callback().retrieve()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdateCallback(TestCallbackBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Callback().update(**self.callback_data_update)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_callback_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Callback().update(**self.callback_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestDeleteCallback(TestCallbackBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Callback().delete(id=self.callback_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Callback().delete(id=self.callback_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_delete_callback_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Callback().delete()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestListCallback(TestCallbackBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_storage_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Callback().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')



API_CONCEPT = 'https://{workspace_api_domain}/{api_type}/{api_version}/{api_sub_url}'


def set_url(workspace_api_domain, api_type, api_version, api_sub_url):
    """

    :param workspace_api_domain:
    :param api_type:
    :param api_version:
    :param api_sub_url:
    :return:
    """
    return API_CONCEPT.format(
            workspace_api_domain=workspace_api_domain,
            api_type=api_type,
            api_version=api_version,
            api_sub_url=api_sub_url
        )



import unittest
import mock
import uiza

from uiza.api_resources.analytic import Analytic
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError,
    ClientException
)


class TestAnalyticBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestAnalyticBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.api_key = 'test api key'


class TestCreateAnalytic(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_analytic_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Analytic().create()
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestRetrieveAnalytic(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_analytic_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Analytic().retrieve(id='test id')
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestUpdateAnalytic(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_analytic_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Analytic().update()
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestDeleteAnalytic(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_delete_analytic_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Analytic().delete(id='test id')
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestListAnalytic(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_analytic_invalid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(ClientException) as context:
            Analytic().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientException')


class TestGetTotalLine(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Analytic().get_total_line(
            start_date='2018-11-01 20:00',
            end_date='2019-11-02 20:00',
            metric='rebuffer_count'
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Analytic().get_total_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_not_start_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_total_line(
                end_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_not_end_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_total_line(
                start_date='2019-11-02 20:00',
                metric='rebuffer_count'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_total_line_invalid_with_not_metric(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_total_line(
                end_date='2019-11-02 20:00',
                start_date='2019-11-02 20:00'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestGetType(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Analytic().get_type(
            start_date='2018-11-01 20:00',
            end_date='2019-11-02 20:00',
            type_filter='country'
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Analytic().get_type(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_not_start_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_type(
                end_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_not_end_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_type(
                start_date='2019-11-02 20:00',
                type_filter='country'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_type_invalid_with_not_type_filter(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_type(
                end_date='2019-11-02 20:00',
                start_date='2019-11-02 20:00'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestGetLine(TestAnalyticBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Analytic().get_line(
            start_date='2018-11-01 20:00',
            end_date='2019-11-02 20:00',
            type='video_startup_time'
        )
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Analytic().get_line(
                start_date='2018-11-01 20:00',
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_not_start_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_line(
                end_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_not_end_date(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_line(
                start_date='2019-11-02 20:00',
                type='video_startup_time'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_line_invalid_with_not_type(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Analytic().get_line(
                start_date='2019-11-02 20:00',
                end_date='2019-11-02 20:00'
            )
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')



import uiza
from uiza import Connection
from uiza.api_resources.base.base import UizaBase
from uiza.settings.config import settings
from uiza.utility.utility import set_url


class Category(UizaBase):

    def __init__(self):
        self.connection = Connection(workspace_api_domain=uiza.workspace_api_domain, api_key=uiza.authorization)
        self.connection.url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.category.type,
            api_version=settings.uiza_api.category.version,
            api_sub_url=settings.uiza_api.category.sub_url
        )
        self.category_relation_url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.category.type,
            api_version=settings.uiza_api.category.version,
            api_sub_url='media/entity/related/metadata'
        )

    def create_relation(self, entity_id, metadata_ids):
        """
        Create relation for entity and category
        :param entity_id: identifier of entity
        :param metadata_ids: identifier of category
        :return: tuple of response data and status code
        """
        self.connection.url = self.category_relation_url
        data_body = dict(
            entityId=entity_id,
            metadataIds=metadata_ids
        )
        result = self.connection.post(data_body)

        return result

    def delete_relation(self, entity_id, metadata_ids):
        """
        Delete relation for entity and category
        :param entity_id: identifier of entity
        :param metadata_ids: identifier of category
        :return: tuple of response data and status code
        """
        self.connection.url = self.category_relation_url
        data_body = dict(
            entityId=entity_id,
            metadataIds=metadata_ids
        )
        result = self.connection.delete(data_body)

        return result



import uiza
from uiza import Connection
from uiza.api_resources.base.base import UizaBase
from uiza.settings.config import settings
from uiza.utility.utility import set_url
from uiza.exceptions import ClientException


class Analytic(UizaBase):

    def __init__(self):
        self.connection = Connection(workspace_api_domain=uiza.workspace_api_domain, api_key=uiza.authorization)
        self.connection.url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.analytic.type,
            api_version=settings.uiza_api.analytic.version,
            api_sub_url=settings.uiza_api.analytic.sub_url
        )

    def list(self, **params):
        """
        Override method list of Uizabase
        :param params:
        :return: Raise error when get method list
        """
        raise ClientException('Callback list method not found')

    def create(self, **data):
        """
        Override method create of Uizabase
        :param data:
        :return: Raise error when get method create
        """
        raise ClientException('Callback create method not found')

    def update(self, **kwargs):
        """
        Override method update of Uizabase
        :param kwargs:
        :return: Raise error when get method update
        """
        raise ClientException('Callback update method not found')

    def retrieve(self, id):
        """
        Override method retrieve of Uizabase
        :param id:
        :return: Raise error when get method retrieve
        """
        raise ClientException('Callback retrieve method not found')

    def delete(self, id):
        """
        Override method delete of Uizabase
        :return: Raise error when get method delete
        """
        raise ClientException('Callback delete method not found')

    def get_total_line(self, start_date, end_date, metric):
        """
        Get total line
        :param start_date: start date
        :param end_date: end date
        :param metric: get metric from https://docs.uiza.io/#analytic-metrics
        :return:
        """
        params = dict(
            start_date=start_date,
            end_date=end_date,
            metric=metric
        )
        self.connection.url = '{}/total-line-v2'.format(self.connection.url)
        query = self.url_encode(params=params)
        result = self.connection.get(query=query)

        return result

    def get_type(self, start_date, end_date, type_filter):
        """
        Get data base on 4 type of filter: country, device, title, player
        :param start_date: start date
        :param end_date: end date
        :param type_filter: filter are country, device, title, player
        :return:
        """
        params = dict(
            start_date=start_date,
            end_date=end_date,
            type_filter=type_filter
        )
        self.connection.url = '{}/type'.format(self.connection.url)
        query = self.url_encode(params=params)
        result = self.connection.get(query=query)

        return result

    def get_line(self, start_date, end_date, type):
        """
        Get data grouped by hour
        :param start_date: start date
        :param end_date: end date
        :param type: type
        :return:
        """
        params = dict(
            start_date=start_date,
            end_date=end_date,
            type=type
        )
        self.connection.url = '{}/line'.format(self.connection.url)
        query = self.url_encode(params=params)
        result = self.connection.get(query=query)

        return result



try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


class UizaBase(object):

    data_validated = None
    connection = None

    def create(self, **data):
        """
        Create data
        :param data: data body will be created
        :return: tuple of data created and status code
        """
        result = self.connection.post(data=data)
        try:
            query = self.url_encode(params={'id': result[0].id})
            result = self.connection.get(query=query)
        except Exception:
            pass

        return result

    def update(self, **kwargs):
        """
        Update data
        :param kwargs: data body will be updated
        :return: tuple of data updated and status code
        """
        result = self.connection.put(data=kwargs)
        try:
            query = self.url_encode(params={'id': result[0].id})
            result = self.connection.get(query=query)
        except Exception:
            pass

        return result

    def list(self, **params):
        """
        List data
        :param params: params
        :return: tuple of data exist and status code
        """
        query = ''
        if params:
            query = self.url_encode(params=params)
        result = self.connection.get(query=query)

        return result

    def retrieve(self, id):
        """
        Get detail
        :param id: id of object
        :return: tuple of data and status code
        """
        query = self.url_encode(params={'id': id})
        result = self.connection.get(query=query)

        return result

    def delete(self, id):
        """ata
        Delete data
        :return: tuple of id removed and status code
        """
        result = self.connection.delete(dict(id=id))

        return result

    def url_encode(self, params):
        return '?{}'.format(urlencode(params))



import uiza
from uiza import Connection
from uiza.api_resources.base.base import UizaBase
from uiza.settings.config import settings
from uiza.utility.utility import set_url
from uiza.exceptions import ClientException


class Callback(UizaBase):

    def __init__(self):
        self.connection = Connection(workspace_api_domain=uiza.workspace_api_domain, api_key=uiza.authorization)
        self.connection.url = set_url(
            workspace_api_domain=self.connection.workspace_api_domain,
            api_type=settings.uiza_api.callback.type,
            api_version=settings.uiza_api.callback.version,
            api_sub_url=settings.uiza_api.callback.sub_url
        )

    def list(self, **params):
        """
        Override method list of Uizabase
        :param params:
        :return: Raise error when get method list
        """
        raise ClientException('Callback list method not found')



import unittest
import mock
import uiza

from uiza.api_resources.entity import Entity
from uiza.exceptions import (
    BadRequestError,
    UnauthorizedError,
    NotFoundError,
    UnprocessableError,
    InternalServerError,
    ServiceUnavailableError,
    ClientError,
    ServerError
)


class TestEntityBaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestEntityBaseTestCase, self).__init__(*args, **kwargs)
        uiza.workspace_api_domain = 'test domain'
        uiza.authorization = 'test api key'
        self.entity_id = '37d6706e-be91-463e-b3b3-b69451dd4752'
        self.entity_data_create = {
            "name": "Sample Video Python",
            "url": "https://example.com/video.mp4",
            "inputType": "http",
            "description": "Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry'\''s standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like Aldus PageMaker including versions of Lorem Ipsum.",
            "shortDescription": "Lorem Ipsum is simply dummy text of the printing and typesetting industry",
            "poster": "https://example.com/picture001.jpeg",
            "thumbnail": "https://example.com/picture002.jpeg",
            "extendMetadata": {
                "movie_category": "action",
                "imdb_score": 8.8,
                "published_year": "2018"
            },
            "embedMetadata": {
                "artist": "John Doe",
                "album": "Album sample",
                "genre": "Pop"
            },
            "metadataIds": ["16a9e425-efb0-4360-bd92-8d49da111e88"]
        }
        self.entity_data_update = dict(id=self.entity_id, name='Test update')


class TestCreateEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().create(**self.entity_data_create)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_create_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().create(**self.entity_data_create)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestRetrieveEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().retrieve(id=self.entity_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().retrieve(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_retrieve_entity_invalid_with_not_entity_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            Entity().retrieve()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestUpdateEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().update(**self.entity_data_update)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_update_storage_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().update(**self.entity_data_update)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestDeleteEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().delete(id=self.entity_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().delete(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_not_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Entity().delete()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestListEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().list()
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_list_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().list()
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')


class TestSearchEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().search(keyword='Test keyword')
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().search(keyword='Test keyword')
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_search_entity_invalid_with_not_keyword(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Entity().search()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestPublishEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().publish(self.entity_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().publish(self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_publish_entity_invalid_with_not_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(TypeError) as context:
            Entity().publish()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestGetPublishStatusEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().get_status_publish(id=self.entity_id)
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().get_status_publish(id=self.entity_id)
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_publish_status_invalid_with_not_id(self, mock_request_http):
        mock_request_http.return_value = True, 200
        with self.assertRaises(Exception) as context:
            Entity().get_status_publish()
        self.assertTrue(context.exception.__class__.__name__, 'TypeError')


class TestGetAWSUploadKeyEntity(TestEntityBaseTestCase):

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_valid(self, mock_request_http):
        mock_request_http.return_value = True, 200
        data = Entity().get_aws_upload_key()
        self.assertEqual(data[1], 200)

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_400(self, mock_request_http):
        mock_request_http.return_value = True, 400
        with self.assertRaises(BadRequestError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'BadRequestError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_401(self, mock_request_http):
        mock_request_http.return_value = True, 401
        with self.assertRaises(UnauthorizedError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'UnauthorizedError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_404(self, mock_request_http):
        mock_request_http.return_value = True, 404
        with self.assertRaises(NotFoundError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'NotFoundError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_422(self, mock_request_http):
        mock_request_http.return_value = True, 422
        with self.assertRaises(UnprocessableError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'UnprocessableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_500(self, mock_request_http):
        mock_request_http.return_value = True, 500
        with self.assertRaises(InternalServerError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'InternalServerError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_503(self, mock_request_http):
        mock_request_http.return_value = True, 503
        with self.assertRaises(ServiceUnavailableError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'ServiceUnavailableError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_4xx(self, mock_request_http):
        mock_request_http.return_value = True, 412
        with self.assertRaises(ClientError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'ClientError')

    @mock.patch('uiza.Connection._request_http')
    def test_get_aws_upload_key_invalid_with_status_code_5xx(self, mock_request_http):
        mock_request_http.return_value = True, 512
        with self.assertRaises(ServerError) as context:
            Entity().get_aws_upload_key()
        self.assertTrue(context.exception.__class__.__name__, 'ServerError')
